@@ -4,22 +4,40 @@ use strict;
 use warnings;
 
 use Carp qw(croak carp longmess);
+use HTTP::Status qw(:constants);
+use Plack::Response;
 use Plack::Util;
 
 use Raisin::Request;
-use Raisin::Response;
 use Raisin::Routes;
 use Raisin::Util;
 
-our $VERSION = '0.69';
+use Raisin::Middleware::Formatter;
+use Raisin::Encoder;
+use Raisin::Decoder;
+
+use Plack::Util::Accessor qw(
+    middleware
+    mounted
+    routes
+
+    decoder
+    encoder
+);
+
+our $VERSION = '0.70';
 
 sub new {
     my ($class, %args) = @_;
+
     my $self = bless { %args }, $class;
 
-    $self->{routes} = Raisin::Routes->new;
-    $self->{mounted} = [];
-    $self->{middleware} = {};
+    $self->middleware({});
+    $self->mounted([]);
+    $self->routes(Raisin::Routes->new);
+
+    $self->decoder(Raisin::Decoder->new);
+    $self->encoder(Raisin::Encoder->new);
 
     $self;
 }
@@ -46,10 +64,8 @@ sub add_middleware {
 }
 
 # Routes
-sub routes { shift->{routes} }
 sub add_route {
     my ($self, %params) = @_;
-    $params{api_format} = $self->api_format if $self->api_format;
     $self->routes->add(%params);
 }
 
@@ -88,6 +104,14 @@ sub run {
         $psgi = $mw->wrap($psgi, @$args);
     }
 
+    $psgi = Raisin::Middleware::Formatter->wrap(
+        $psgi,
+        default_format => $self->default_format,
+        format => $self->format,
+        decoder => $self->decoder,
+        encoder => $self->encoder
+    );
+
     # load fallback logger (Raisin::Logger)
     $self->load_plugin('Logger', fallback => 1);
 
@@ -122,11 +146,11 @@ sub generate_allowed_methods {
 sub psgi {
     my ($self, $env) = @_;
 
-    # Diffrent for each response
-    my $req = $self->req(Raisin::Request->new($self, $env));
-    my $res = $self->res(Raisin::Response->new($self));
+    # New for each response
+    my $req = $self->req(Raisin::Request->new($env));
+    my $res = $self->res(Plack::Response->new);
 
-    # Build API docs
+    # Generate API description
     if ($self->can('swagger_build_spec')) {
         $self->swagger_build_spec;
     }
@@ -138,29 +162,22 @@ sub psgi {
         my $route = $self->routes->find($req->method, $req->path);
         # The requested path exists but requested method not
         if (!$route && $self->{allowed_methods}{ $req->path }) {
-            $res->status(405); # Method Not Allowed
+            $res->status(HTTP_METHOD_NOT_ALLOWED);
             return $res->finalize;
         }
         # Nothing found
         elsif (!$route) {
-            $res->status(404);
-            return $res->finalize;
-        }
-
-        my $type = $req->accept_format;
-        my $accept = $req->header('Accept');
-        if (   ($accept && $accept ne '*/*' && !$type)
-            || (($self->api_format && $type) && ($self->api_format ne $type)))
-        {
-            $self->log(error => 'Content-type provided not acceptable');
-            $res->status(406);
+            $res->status(HTTP_NOT_FOUND);
             return $res->finalize;
         }
 
         my $code = $route->code;
         if (!$code || ($code && ref($code) ne 'CODE')) {
-            $res->status(500);
-            $res->body('Invalid Endpoint For ' . $req->path);
+            $self->log(error => "route ${ \$req->path } returns nothing or not CODE");
+
+            $res->status(HTTP_INTERNAL_SERVER_ERROR);
+            $res->body('Internal error');
+
             return $res->finalize;
         }
 
@@ -168,34 +185,24 @@ sub psgi {
 
         # Validation and coercion of declared params
         if (!$req->prepare_params($route->params, $route->named)) {
-            $res->status(400);
+            $res->status(HTTP_BAD_REQUEST);
             $res->body('Invalid Parameters');
             return $res->finalize;
         }
 
         $self->hook('after_validation')->($self);
 
-        # Evaluate a user's endpoint
+        # Evaluate an endpoint
         my $data = $code->($req->declared_params);
         if (defined $data) {
-            # Delayed responses
+            # Delayed response
             return $data if ref($data) eq 'CODE';
 
             $res->body($data);
         }
 
-        if (!$res->rendered) {
-            my $format = $route->format || $req->header('Accept');
-            $res->format($format);
-            $res->render;
-        }
-
         $self->hook('after')->($self);
 
-        if (!$res->rendered) {
-            $self->log(error => 'Nothing rendered');
-        }
-
         1;
     } or do {
         my $e = longmess($@);
@@ -204,7 +211,7 @@ sub psgi {
         my $msg = $ENV{PLACK_ENV}
             && $ENV{PLACK_ENV} eq 'deployment' ? 'Internal Error' : $e;
 
-        $res->status(500);
+        $res->status(HTTP_INTERNAL_SERVER_ERROR);
         $res->body($msg);
     };
 
@@ -218,6 +225,8 @@ sub psgi {
 # Finalize response
 sub before_finalize {
     my $self = shift;
+
+    $self->res->status(HTTP_OK) unless $self->res->status;
     $self->res->header('X-Framework' => "Raisin $VERSION");
 
     if ($self->api_version) {
@@ -232,28 +241,32 @@ sub finalize {
 }
 
 # Application defaults
-sub api_default_format {
+sub default_format {
     my ($self, $format) = @_;
 
     if ($format) {
-        $self->{api_default_format} = Raisin::Util::make_serializer_class($format);
+        $self->{default_format} = $format;
     }
 
-    $self->{api_default_format} || Raisin::Util::make_serializer_class('yaml');
+    $self->{default_format} || 'yaml';
 }
 
-sub api_format {
+sub format {
     my ($self, $format) = @_;
 
-    if ($format && grep { lc($format) eq $_ } qw(json yaml)) {
-        $self->{api_format} = lc $format;
-        $self->api_default_format(lc $format);
-    }
-    elsif ($format) {
-        carp "Can't use specified format. Currently supported only JSON and YAML.";
+    if ($format) {
+        my @decoders = keys %{ $self->decoder->all };
+
+        if (grep { lc($format) eq $_ } @decoders) {
+            $self->{format} = lc $format;
+            $self->default_format(lc $format);
+        }
+        else {
+            carp 'Invalid format, choose one of: ', join(', ', @decoders);
+        }
     }
 
-    $self->{api_format};
+    $self->{format};
 }
 
 sub api_version {
@@ -275,8 +288,6 @@ sub res {
     $self->{res};
 }
 
-sub param { shift->req->parameters->mixed }
-
 sub session {
     my $self = shift;
 
@@ -299,14 +310,7 @@ Raisin - a REST API micro framework for Perl.
 
 =head1 SYNOPSIS
 
-    use strict;
-    use warnings;
-
-    use utf8;
-
-    use FindBin;
-    use lib "$FindBin::Bin/../../lib";
-
+    use HTTP::Status qw(:constants);
     use List::Util qw(max);
     use Raisin::API;
     use Types::Standard qw(HashRef Any Int Str);
@@ -326,8 +330,12 @@ Raisin - a REST API micro framework for Perl.
         },
     );
 
-    plugin 'Swagger', enable => 'CORS';
-    #api_format 'json';
+    middleware 'CrossOrigin',
+        origins => '*',
+        methods => [qw/DELETE GET HEAD OPTIONS PATCH POST PUT/],
+        headers => [qw/accept authorization content-type api_key_token/];
+
+    plugin 'Swagger';
 
     swagger_setup(
         title => 'A POD synopsis API',
@@ -391,6 +399,7 @@ Raisin - a REST API micro framework for Perl.
             my $id = max(keys %USERS) + 1;
             $USERS{$id} = $params->{user};
 
+            res->status(HTTP_CREATED);
             { success => 1 }
         };
 
@@ -406,7 +415,9 @@ Raisin - a REST API micro framework for Perl.
             summary 'Delete user';
             del sub {
                 my $params = shift;
-                { success => delete $USERS{ $params->{id} } };
+                delete $USERS{ $params->{id} };
+                res->status(HTTP_NO_CONTENT);
+                undef;
             };
         };
     };
@@ -435,7 +446,8 @@ Adds a route to an application.
 
 =head3 route_param
 
-Define a route parameter as a namespace C<route_param>.
+Defines a route parameter as a resource C<id> which can be anything if type
+isn't specified for it.
 
     route_param id => sub { ... };
 
@@ -464,8 +476,8 @@ Shortcuts to add a C<route> restricted to the corresponding HTTP method.
 
 =head3 desc
 
-Can be applied to C<resource> or any of the HTTP method to add a verbose
-explanation for an operation or for a resource.
+Adds a description to C<resource> or any of the HTTP methods.
+Useful for OpenAPI as it's shown there as a description of an action.
 
     desc 'Some long explanation about an action';
     put sub { ... };
@@ -475,28 +487,25 @@ explanation for an operation or for a resource.
 
 =head3 summary
 
-Can be applied to any of the HTTP method to add a short summary of
-what the operation does.
+Same as L</desc> but shorter.
 
     summary 'Some summary';
     put sub { ... };
 
 =head3 tags
 
-A list of tags for API documentation control.
 Tags can be used for logical grouping of operations by resources
-or any other qualifier.
+or any other qualifier. Using in API description.
 
     tags 'delete', 'user';
     delete sub { ... };
 
 By default tags are added automatically based on it's namespace but you always
-can overwrite it using a C<tags> function.
+can overwrite it using the function.
 
 =head3 entity
 
-Entity keyword allows to describe response object which will be used to generate
-OpenAPI specification.
+Describes response object which will be used to generate OpenAPI description.
 
     entity 'MusicApp::Entity::Album';
     get {
@@ -508,7 +517,7 @@ OpenAPI specification.
 =head3 params
 
 Defines validations and coercion options for your parameters.
-Can be applied to any HTTP method and/or C<route_param> to describe parameters.
+Can be applied to any HTTP method and/or L</route_param> to describe parameters.
 
     params(
         requires('name', type => Str),
@@ -526,9 +535,9 @@ For more see L<Raisin/Validation-and-coercion>.
 
 =head3 api_default_format
 
-Specifies default API format mode when formatter doesn't specified by API user.
-E.g. URI is asked without an extension (C<json>, C<yaml>) or C<Accept> header
-isn't specified.
+Specifies default API format mode when formatter isn't specified by API user.
+E.g. if URI is asked without an extension (C<json>, C<yaml>) or C<Accept> header
+isn't specified the default format will be used.
 
 Default value: C<YAML>.
 
@@ -540,7 +549,9 @@ See also L<Raisin/API-FORMATS>.
 
 Restricts API to use only specified formatter to serialize and deserialize data.
 
-Already exists L<Raisin::Plugin::Format::JSON> and L<Raisin::Plugin::Format::YAML>.
+Already exists L<Raisin::Encoder::JSON>, L<Raisin::Encoder::YAML>,
+and L<Raisin::Encoder::Text>, but you can always register your own
+using L</register_encoder>.
 
     api_format 'json';
 
@@ -557,7 +568,7 @@ Sets up an API version header.
 Loads a Raisin module. A module options may be specified after the module name.
 Compatible with L<Kelp> modules.
 
-    plugin 'Swagger', enable => 'CORS';
+    plugin 'Swagger';
 
 =head3 middleware
 
@@ -585,6 +596,22 @@ In C<RaisinApp.pm>:
 
     1;
 
+=head3 register_decoder
+
+Registers a third-party parser (decoder).
+
+    register_decoder(xml => 'My::Parser::XML');
+
+Also see L<Raisin::Decoder>.
+
+=head3 register_encoder
+
+Registers a third-party formatter (encoder).
+
+    register_encoder(xml => 'My::Formatter::XML');
+
+Also see L<Raisin::Encoder>.
+
 =head3 run
 
 Returns the C<PSGI> application.
@@ -723,7 +750,7 @@ In the case of conflict between either of:
 
 =over
 
-=item * route string parameters;
+=item * path parameters;
 
 =item * GET, POST and PUT parameters;
 
@@ -731,7 +758,7 @@ In the case of conflict between either of:
 
 =back
 
-route string parameters will have precedence.
+Path parameters have precedence.
 
 Query string and body parameters will be merged (see L<Plack::Request/parameters>)
 
@@ -865,7 +892,6 @@ Use C<ArrayRef[*]> types from your compatible type library to define arrays.
 
     requires('list', type => ArrayRef[Int], desc => 'List of integers')
 
-
 =head2 Types
 
 Raisin supports Moo(se)-compatible type constraint so you can use any of the
@@ -923,7 +949,7 @@ C<encode_json> in each C<JSON> API implementation.
 Your API can declare to support only one serializator by using L<Raisin/api_format>.
 
 Custom formatters for existing and additional types can be defined with a
-L<Raisin::Plugin::Format>.
+L<Raisin::Encoder>/L<Raisin::Decoder>.
 
 =over
 
@@ -935,7 +961,7 @@ Call C<JSON::encode_json> and C<JSON::decode_json>.
 
 Call C<YAML::Dump> and C<YAML::Load>.
 
-=item TEXT
+=item Text
 
 Call C<Data::Dumper-E<gt>Dump> if output data is not a string.
 
